@@ -315,6 +315,7 @@ void RequestGroup::processCheckIntegrityEntry(std::deque<Command*>& commands,
   if(e->option->getAsBool(PREF_CHECK_INTEGRITY) &&
      entry->isValidationReady()) {
     entry->initValidator();
+    entry->cutTrailingGarbage();
     CheckIntegrityCommand* command =
       new CheckIntegrityCommand(CUIDCounterSingletonHolder::instance()->newID(), this, e, entry);
     commands.push_back(command);
@@ -478,18 +479,22 @@ bool RequestGroup::tryAutoFileRenaming()
 void RequestGroup::createNextCommandWithAdj(std::deque<Command*>& commands,
 					    DownloadEngine* e, int numAdj)
 {
-  unsigned int numCommand;
-  if(_numConcurrentCommand == 0) {
-    numCommand = _uris.size();
+  int numCommand;
+  if(getTotalLength() == 0) {
+    numCommand = 1+numAdj;
   } else {
-    int n = _numConcurrentCommand+numAdj;
-    if(n > 0) {
-      numCommand = n;
+    if(_numConcurrentCommand == 0) {
+      numCommand = _uris.size();
     } else {
-      return;
+      numCommand = _numConcurrentCommand;
     }
+    numCommand += numAdj;
+    numCommand = std::min(static_cast<int>(_downloadContext->getNumPieces()),
+			  numCommand);
+  }
+  if(numCommand > 0) {
+    createNextCommand(commands, e, numCommand);
   }
-  createNextCommand(commands, e, numCommand, Request::METHOD_GET);
 }
 
 void RequestGroup::createNextCommand(std::deque<Command*>& commands,
@@ -952,6 +957,11 @@ void RequestGroup::removeURIWhoseHostnameIs(const std::string& hostname)
   _uris = newURIs;
 }
 
+void RequestGroup::removeIdenticalURI(const std::string& uri)
+{
+  _uris.erase(std::remove(_uris.begin(), _uris.end(), uri), _uris.end());
+}
+
 void RequestGroup::reportDownloadFinished()
 {
   _logger->notice(MSG_FILE_DOWNLOAD_COMPLETED,
